@@ -3,32 +3,33 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : PLAIN SASL mechanism
 %%% Created :  8 Mar 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: cyrsasl_plain.erl,v 1.2 2003/06/07 17:30:25 aleksey Exp $
+%%% Id      : $Id: cyrsasl_plain.erl 307 2005-04-17 18:08:34Z tmallard $
 %%%----------------------------------------------------------------------
 
 -module(cyrsasl_plain).
 -author('alexey@sevcom.net').
--vsn('$Revision: 1.2 $ ').
+-vsn('$Revision$ ').
 
--export([start/1, stop/0, mech_new/0, mech_step/2, parse/1]).
+-export([start/1, stop/0, mech_new/2, mech_step/2, parse/1]).
 
 -behaviour(cyrsasl).
-%-behaviour(gen_mod).
 
-start(Opts) ->
+-record(state, {check_password}).
+
+start(_Opts) ->
     cyrsasl:register_mechanism("PLAIN", ?MODULE),
     ok.
 
 stop() ->
     ok.
 
-mech_new() ->
-    {ok, []}.
+mech_new(_GetPassword, CheckPassword) ->
+    {ok, #state{check_password = CheckPassword}}.
 
 mech_step(State, ClientIn) ->
     case parse(ClientIn) of
 	[AuthzId, User, Password] ->
-	    case ejabberd_auth:check_password(User, Password) of
+	    case (State#state.check_password)(User, Password) of
 		true ->
 		    {ok, [{username, User}, {authzid, AuthzId}]};
 		_ ->
