@@ -3,12 +3,12 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : Serve C2S connection
 %%% Created : 16 Nov 2002 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: ejabberd_c2s.erl,v 1.84 2004/09/25 20:52:19 aleksey Exp $
+%%% Id      : $Id: ejabberd_c2s.erl 345 2005-05-09 19:12:41Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(ejabberd_c2s).
 -author('alexey@sevcom.net').
--vsn('$Revision: 1.84 $ ').
+-vsn('$Revision: 345 $ ').
 
 -behaviour(gen_fsm).
 
@@ -46,6 +46,7 @@
 		access,
 		shaper,
 		tls = false,
+		tls_required = false,
 		tls_enabled = false,
 		tls_options = [],
 		authentificated = false,
@@ -82,6 +83,10 @@
 	xml:element_to_string(?SERR_INVALID_NAMESPACE)).
 -define(INVALID_XML_ERR,
 	xml:element_to_string(?SERR_XML_NOT_WELL_FORMED)).
+-define(HOST_UNKNOWN_ERR,
+	xml:element_to_string(?SERR_HOST_UNKNOWN)).
+-define(POLICY_VIOLATION_ERR(Lang, Text),
+	xml:element_to_string(?SERRT_POLICY_VIOLATION(Lang, Text))).
 
 %%%----------------------------------------------------------------------
 %%% API
@@ -117,8 +122,9 @@ init([{SockMod, Socket}, Opts]) ->
 		 _ -> none
 	     end,
     StartTLS = lists:member(starttls, Opts),
+    StartTLSRequired = lists:member(starttls_required, Opts),
     TLSEnabled = lists:member(tls, Opts),
-    TLS = StartTLS orelse TLSEnabled,
+    TLS = StartTLS orelse StartTLSRequired orelse TLSEnabled,
     TLSOpts = lists:filter(fun({certfile, _}) -> true;
 			      (_) -> false
 			   end, Opts),
@@ -132,15 +138,16 @@ init([{SockMod, Socket}, Opts]) ->
 		RecPid = ejabberd_receiver:start(Socket, SockMod, none),
 		{SockMod, Socket, RecPid}
 	end,
-    {ok, wait_for_stream, #state{socket      = Socket1,
-				 sockmod     = SockMod1,
-				 receiver    = ReceiverPid,
-				 tls         = TLS,
-				 tls_enabled = TLSEnabled,
-				 tls_options = TLSOpts,
-				 streamid    = new_id(),
-				 access      = Access,
-				 shaper      = Shaper}}.
+    {ok, wait_for_stream, #state{socket       = Socket1,
+				 sockmod      = SockMod1,
+				 receiver     = ReceiverPid,
+				 tls          = TLS,
+				 tls_required = StartTLSRequired,
+				 tls_enabled  = TLSEnabled,
+				 tls_options  = TLSOpts,
+				 streamid     = new_id(),
+				 access       = Access,
+				 shaper       = Shaper}}.
 
 
 %%----------------------------------------------------------------------
@@ -159,80 +166,128 @@ wait_for_stream({xmlstreamstart, _Name, Attrs}, StateData) ->
 		  end,
     case xml:get_attr_s("xmlns:stream", Attrs) of
 	?NS_STREAM ->
-	    Lang = xml:get_attr_s("xml:lang", Attrs),
-	    case xml:get_attr_s("version", Attrs) of
-		"1.0" ->
-		    Header = io_lib:format(?STREAM_HEADER,
-					   [StateData#state.streamid,
-					    ?MYNAME,
-					    " version='1.0'",
-					    DefaultLang]),
-		    send_text(StateData, Header),
-		    case StateData#state.authentificated of
-			false ->
-			    SASLState =
-				cyrsasl:server_new("jabber", ?MYNAME, "", []),
-			    Mechs = lists:map(
-				      fun(S) ->
-					      {xmlelement, "mechanism", [],
-					       [{xmlcdata, S}]}
-				      end, cyrsasl:listmech()),
-			    TLS = StateData#state.tls,
-			    TLSEnabled = StateData#state.tls_enabled,
-			    SockMod = StateData#state.sockmod,
-			    TLSFeature =
-				case (TLS == true) andalso
-				    (TLSEnabled == false) andalso
-				    (SockMod == gen_tcp) of
-				    true ->
-					[{xmlelement, "starttls",
-					  [{"xmlns", ?NS_TLS}], []}];
-				    false ->
-					[]
-				end,
-			    send_element(StateData,
-					 {xmlelement, "stream:features", [],
-					  TLSFeature ++
-					  [{xmlelement, "mechanisms",
-					    [{"xmlns", ?NS_SASL}],
-					    Mechs},
-					   {xmlelement, "register",
-					    [{"xmlns", ?NS_FEATURE_IQREGISTER}],
-					    []}]}),
-			    {next_state, wait_for_feature_request,
-			     StateData#state{sasl_state = SASLState,
-					     lang = Lang}};
-			_ ->
-			    case StateData#state.resource of
-				"" ->
-				    send_element(
-				      StateData,
-				      {xmlelement, "stream:features", [],
-				       [{xmlelement, "bind",
-					 [{"xmlns", ?NS_BIND}], []},
-					{xmlelement, "session",
-					 [{"xmlns", ?NS_SESSION}], []}]}),
-				    {next_state, wait_for_bind,
-				     StateData#state{lang = Lang}};
+	    Server = jlib:nameprep(xml:get_attr_s("to", Attrs)),
+	    case lists:member(Server, ?MYHOSTS) of
+		true ->
+		    Lang = xml:get_attr_s("xml:lang", Attrs),
+		    case xml:get_attr_s("version", Attrs) of
+			"1.0" ->
+			    Header = io_lib:format(?STREAM_HEADER,
+						   [StateData#state.streamid,
+						    Server,
+						    " version='1.0'",
+						    DefaultLang]),
+			    send_text(StateData, Header),
+			    case StateData#state.authentificated of
+				false ->
+				    SASLState =
+					cyrsasl:server_new(
+					  "jabber", Server, "", [],
+					  fun(U) ->
+						  ejabberd_auth:get_password(
+						    U, Server)
+					  end,
+					  fun(U, P) ->
+						  ejabberd_auth:check_password(
+						    U, Server, P)
+					  end),
+				    Mechs = lists:map(
+					      fun(S) ->
+						      {xmlelement, "mechanism", [],
+						       [{xmlcdata, S}]}
+					      end, cyrsasl:listmech()),
+				    TLS = StateData#state.tls,
+				    TLSEnabled = StateData#state.tls_enabled,
+				    TLSRequired = StateData#state.tls_required,
+				    SockMod = StateData#state.sockmod,
+				    TLSFeature =
+					case (TLS == true) andalso
+					    (TLSEnabled == false) andalso
+					    (SockMod == gen_tcp) of
+					    true ->
+						case TLSRequired of
+						    true ->
+							[{xmlelement, "starttls",
+							  [{"xmlns", ?NS_TLS}],
+							  [{xmlelement, "required",
+							    [], []}]}];
+						    _ ->
+							[{xmlelement, "starttls",
+							  [{"xmlns", ?NS_TLS}], []}]
+						end;
+					    false ->
+						[]
+					end,
+				    send_element(StateData,
+						 {xmlelement, "stream:features", [],
+						  TLSFeature ++
+						  [{xmlelement, "mechanisms",
+						    [{"xmlns", ?NS_SASL}],
+						    Mechs},
+						   {xmlelement, "register",
+						    [{"xmlns", ?NS_FEATURE_IQREGISTER}],
+						    []}]}),
+				    {next_state, wait_for_feature_request,
+				     StateData#state{server = Server,
+						     sasl_state = SASLState,
+						     lang = Lang}};
 				_ ->
-				    send_element(
-				      StateData,
-				      {xmlelement, "stream:features", [], []}),
-				    {next_state, wait_for_session,
-				     StateData#state{lang = Lang}}
+				    case StateData#state.resource of
+					"" ->
+					    send_element(
+					      StateData,
+					      {xmlelement, "stream:features", [],
+					       [{xmlelement, "bind",
+						 [{"xmlns", ?NS_BIND}], []},
+						{xmlelement, "session",
+						 [{"xmlns", ?NS_SESSION}], []}]}),
+					    {next_state, wait_for_bind,
+					     StateData#state{server = Server,
+							     lang = Lang}};
+					_ ->
+					    send_element(
+					      StateData,
+					      {xmlelement, "stream:features", [], []}),
+					    {next_state, wait_for_session,
+					     StateData#state{server = Server,
+							     lang = Lang}}
+				    end
+			    end;
+			_ ->
+			    Header = io_lib:format(
+				       ?STREAM_HEADER,
+				       [StateData#state.streamid, Server, "",
+					DefaultLang]),
+			    if
+				(not StateData#state.tls_enabled) and
+				StateData#state.tls_required ->
+				    send_text(StateData,
+					      Header ++
+					      ?POLICY_VIOLATION_ERR(
+						 Lang,
+						 "Use of STARTTLS required") ++
+					      ?STREAM_TRAILER),
+				    {stop, normal, StateData};
+				true ->
+				    send_text(StateData, Header),
+				    {next_state, wait_for_auth, 
+				     StateData#state{server = Server,
+						     lang = Lang}}
 			    end
 		    end;
 		_ ->
 		    Header = io_lib:format(
 			       ?STREAM_HEADER,
-			       [StateData#state.streamid, ?MYNAME, "", DefaultLang]),
-		    send_text(StateData, Header),
-		    {next_state, wait_for_auth, StateData#state{lang = Lang}}
+			       [StateData#state.streamid, ?MYNAME, "",
+				DefaultLang]),
+		    send_text(StateData,
+			      Header ++ ?HOST_UNKNOWN_ERR ++ ?STREAM_TRAILER),
+		    {stop, normal, StateData}
 	    end;
 	_ ->
 	    Header = io_lib:format(
 		       ?STREAM_HEADER,
-		       [StateData#state.streamid, ?MYNAME, "", ""]),
+		       [StateData#state.streamid, ?MYNAME, "", DefaultLang]),
 	    send_text(StateData,
 		      Header ++ ?INVALID_NS_ERR ++ ?STREAM_TRAILER),
 	    {stop, normal, StateData}
@@ -283,30 +338,35 @@ wait_for_auth({xmlstreamelement, El}, StateData) ->
 	    send_element(StateData, Err),
 	    {next_state, wait_for_auth, StateData};
 	{auth, _ID, set, {U, P, D, R}} ->
-	    io:format("AUTH: ~p~n", [{U, P, D, R}]),
 	    JID = jlib:make_jid(U, StateData#state.server, R),
 	    case (JID /= error) andalso
 		(acl:match_rule(StateData#state.access, JID) == allow) of
 		true ->
 		    case ejabberd_auth:check_password(
-			   U, P, StateData#state.streamid, D) of
+			   U, StateData#state.server, P,
+			   StateData#state.streamid, D) of
 			true ->
 			    ?INFO_MSG(
-			       "(~w) Accepted legacy authentification for ~s",
+			       "(~w) Accepted legacy authentication for ~s",
 			       [StateData#state.socket,
 				jlib:jid_to_string(JID)]),
-			    ejabberd_sm:open_session(U, R),
+			    ejabberd_sm:open_session(
+			      U, StateData#state.server, R),
 			    Res1 = jlib:make_result_iq_reply(El),
 			    Res = setelement(4, Res1, []),
 			    send_element(StateData, Res),
 			    change_shaper(StateData, JID),
-			    {Fs, Ts} = mod_roster:get_subscription_lists(U),
+			    {Fs, Ts} = ejabberd_hooks:run_fold(
+					 roster_get_subscription_lists,
+					 {[], []},
+					 [U, StateData#state.server]),
 			    LJID = jlib:jid_tolower(
 				     jlib:jid_remove_resource(JID)),
 			    Fs1 = [LJID | Fs],
 			    Ts1 = [LJID | Ts],
 			    PrivList =
-				case catch mod_privacy:get_user_list(U) of
+				case catch mod_privacy:get_user_list(
+					     U, StateData#state.server) of
 				    {'EXIT', _} -> none;
 				    PL -> PL
 				end,
@@ -319,7 +379,7 @@ wait_for_auth({xmlstreamelement, El}, StateData) ->
 					     privacy_list = PrivList}};
 			_ ->
 			    ?INFO_MSG(
-			       "(~w) Failed legacy authentification for ~s",
+			       "(~w) Failed legacy authentication for ~s",
 			       [StateData#state.socket,
 				jlib:jid_to_string(JID)]),
 			    Err = jlib:make_error_reply(
@@ -331,7 +391,7 @@ wait_for_auth({xmlstreamelement, El}, StateData) ->
 		    if
 			JID == error ->
 			    ?INFO_MSG(
-			       "(~w) Forbidden legacy authentification for "
+			       "(~w) Forbidden legacy authentication for "
 			       "username '~s' with resource '~s'",
 			       [StateData#state.socket, U, R]),
 			    Err = jlib:make_error_reply(El, ?ERR_JID_MALFORMED),
@@ -339,7 +399,7 @@ wait_for_auth({xmlstreamelement, El}, StateData) ->
 			    {next_state, wait_for_auth, StateData};
 			true ->
 			    ?INFO_MSG(
-			       "(~w) Forbidden legacy authentification for ~s",
+			       "(~w) Forbidden legacy authentication for ~s",
 			       [StateData#state.socket,
 				jlib:jid_to_string(JID)]),
 			    Err = jlib:make_error_reply(El, ?ERR_NOT_ALLOWED),
@@ -351,8 +411,10 @@ wait_for_auth({xmlstreamelement, El}, StateData) ->
 	    case jlib:iq_query_info(El) of
 		#iq{xmlns = ?NS_REGISTER} = IQ ->
 		    ResIQ = mod_register:process_iq(
-			      {"", "", ""}, {"", ?MYNAME, ""}, IQ),
-		    Res1 = jlib:replace_from_to({"", ?MYNAME, ""},
+			      {"", "", ""},
+			      jlib:make_jid("", StateData#state.server, ""),
+			      IQ),
+		    Res1 = jlib:replace_from_to({"", StateData#state.server, ""},
 						{"", "", ""},
 						jlib:iq_to_xml(ResIQ)),
 		    Res = jlib:remove_attr("to", Res1),
@@ -379,9 +441,10 @@ wait_for_feature_request({xmlstreamelement, El}, StateData) ->
     {xmlelement, Name, Attrs, Els} = El,
     TLS = StateData#state.tls,
     TLSEnabled = StateData#state.tls_enabled,
+    TLSRequired = StateData#state.tls_required,
     SockMod = StateData#state.sockmod,
     case {xml:get_attr_s("xmlns", Attrs), Name} of
-	{?NS_SASL, "auth"} ->
+	{?NS_SASL, "auth"} when not ((SockMod == gen_tcp) and TLSRequired) ->
 	    Mech = xml:get_attr_s("mechanism", Attrs),
 	    ClientIn = jlib:decode_base64(xml:get_cdata(Els)),
 	    case cyrsasl:server_start(StateData#state.sasl_state,
@@ -393,10 +456,11 @@ wait_for_feature_request({xmlstreamelement, El}, StateData) ->
 				 {xmlelement, "success",
 				  [{"xmlns", ?NS_SASL}], []}),
 		    U = xml:get_attr_s(username, Props),
-		    ?INFO_MSG("(~w) Accepted authentification for ~s",
+		    ?INFO_MSG("(~w) Accepted authentication for ~s",
 			      [StateData#state.socket, U]),
 		    {next_state, wait_for_stream,
-		     StateData#state{authentificated = true,
+		     StateData#state{streamid = new_id(),
+				     authentificated = true,
 				     user = U
 				    }};
 		{continue, ServerOut, NewSASLState} ->
@@ -426,21 +490,34 @@ wait_for_feature_request({xmlstreamelement, El}, StateData) ->
 	    {next_state, wait_for_stream,
 	     StateData#state{sockmod = tls,
 			     socket = TLSSocket,
+			     streamid = new_id(),
 			     tls_enabled = true
 			    }};
 	_ ->
-	    case jlib:iq_query_info(El) of
-		#iq{xmlns = ?NS_REGISTER} = IQ ->
-		    ResIQ = mod_register:process_iq(
-			      {"", "", ""}, {"", ?MYNAME, ""}, IQ),
-		    Res1 = jlib:replace_from_to({"", ?MYNAME, ""},
-						{"", "", ""},
-						jlib:iq_to_xml(ResIQ)),
-		    Res = jlib:remove_attr("to", Res1),
-		    send_element(StateData, Res),
-		    {next_state, wait_for_feature_request, StateData};
-		_ ->
-		    {next_state, wait_for_feature_request, StateData}
+	    if
+		(SockMod == gen_tcp) and TLSRequired ->
+		    Lang = StateData#state.lang,
+		    send_text(StateData, ?POLICY_VIOLATION_ERR(
+					    Lang,
+					    "Use of STARTTLS required") ++
+					 ?STREAM_TRAILER),
+		    {stop, normal, StateData};
+		true ->
+		    case jlib:iq_query_info(El) of
+			#iq{xmlns = ?NS_REGISTER} = IQ ->
+			    ResIQ = mod_register:process_iq(
+				      {"", "", ""},
+				      jlib:make_jid("", StateData#state.server, ""),
+				      IQ),
+			    Res1 = jlib:replace_from_to({"", StateData#state.server, ""},
+							{"", "", ""},
+							jlib:iq_to_xml(ResIQ)),
+			    Res = jlib:remove_attr("to", Res1),
+			    send_element(StateData, Res),
+			    {next_state, wait_for_feature_request, StateData};
+			_ ->
+			    {next_state, wait_for_feature_request, StateData}
+		    end
 	    end
     end;
 
@@ -469,10 +546,11 @@ wait_for_sasl_response({xmlstreamelement, El}, StateData) ->
 				 {xmlelement, "success",
 				  [{"xmlns", ?NS_SASL}], []}),
 		    U = xml:get_attr_s(username, Props),
-		    ?INFO_MSG("(~w) Accepted authentification for ~s",
+		    ?INFO_MSG("(~w) Accepted authentication for ~s",
 			      [StateData#state.socket, U]),
 		    {next_state, wait_for_stream,
-		     StateData#state{authentificated = true,
+		     StateData#state{streamid = new_id(),
+				     authentificated = true,
 				     user = U
 				    }};
 		{continue, ServerOut, NewSASLState} ->
@@ -494,8 +572,10 @@ wait_for_sasl_response({xmlstreamelement, El}, StateData) ->
 	    case jlib:iq_query_info(El) of
 		#iq{xmlns = ?NS_REGISTER} = IQ ->
 		    ResIQ = mod_register:process_iq(
-			      {"", "", ""}, {"", ?MYNAME, ""}, IQ),
-		    Res1 = jlib:replace_from_to({"", ?MYNAME, ""},
+			      {"", "", ""},
+			      jlib:make_jid("", StateData#state.server, ""),
+			      IQ),
+		    Res1 = jlib:replace_from_to({"", StateData#state.server, ""},
 						{"", "", ""},
 						jlib:iq_to_xml(ResIQ)),
 		    Res = jlib:remove_attr("to", Res1),
@@ -570,23 +650,27 @@ wait_for_session({xmlstreamelement, El}, StateData) ->
 	#iq{type = set, xmlns = ?NS_SESSION} ->
 	    U = StateData#state.user,
 	    R = StateData#state.resource,
-	    io:format("SASLAUTH: ~p~n", [{U, R}]),
 	    JID = StateData#state.jid,
 	    case acl:match_rule(StateData#state.access, JID) of
 		allow ->
 		    ?INFO_MSG("(~w) Opened session for ~s",
 			      [StateData#state.socket,
 			       jlib:jid_to_string(JID)]),
-		    ejabberd_sm:open_session(U, R),
+		    ejabberd_sm:open_session(
+		      U, StateData#state.server, R),
 		    Res = jlib:make_result_iq_reply(El),
 		    send_element(StateData, Res),
 		    change_shaper(StateData, JID),
-		    {Fs, Ts} = mod_roster:get_subscription_lists(U),
+		    {Fs, Ts} = ejabberd_hooks:run_fold(
+				 roster_get_subscription_lists,
+				 {[], []},
+				 [U, StateData#state.server]),
 		    LJID = jlib:jid_tolower(jlib:jid_remove_resource(JID)),
 		    Fs1 = [LJID | Fs],
 		    Ts1 = [LJID | Ts],
 		    PrivList =
-			case catch mod_privacy:get_user_list(U) of
+			case catch mod_privacy:get_user_list(
+				     U, StateData#state.server) of
 			    {'EXIT', _} -> none;
 			    PL -> PL
 			end,
@@ -801,6 +885,7 @@ handle_info({route, From, To, Packet}, StateName, StateData) ->
 %-ifdef(PRIVACY_SUPPORT).
 			case catch mod_privacy:check_packet(
 				     StateData#state.user,
+				     StateData#state.server,
 				     StateData#state.privacy_list,
 				     {From, To, Packet},
 				     in) of
@@ -850,6 +935,7 @@ handle_info({route, From, To, Packet}, StateName, StateData) ->
 		    #iq{} ->
 			case catch mod_privacy:check_packet(
 				     StateData#state.user,
+				     StateData#state.server,
 				     StateData#state.privacy_list,
 				     {From, To, Packet},
 				     in) of
@@ -871,6 +957,7 @@ handle_info({route, From, To, Packet}, StateName, StateData) ->
 	    "message" ->
 		case catch mod_privacy:check_packet(
 			     StateData#state.user,
+			     StateData#state.server,
 			     StateData#state.privacy_list,
 			     {From, To, Packet},
 			     in) of
@@ -921,6 +1008,7 @@ terminate(_Reason, StateName, StateData) ->
 			      [{xmlelement, "status", [],
 				[{xmlcdata, "Replaced by new connection"}]}]},
 		    ejabberd_sm:unset_presence(StateData#state.user,
+					       StateData#state.server,
 					       StateData#state.resource,
 					       "Replaced by new connection"),
 		    presence_broadcast(
@@ -932,6 +1020,7 @@ terminate(_Reason, StateName, StateData) ->
 			      [StateData#state.socket,
 			       jlib:jid_to_string(StateData#state.jid)]),
 		    ejabberd_sm:close_session(StateData#state.user,
+					      StateData#state.server,
 					      StateData#state.resource),
 
 		    Tmp = ?SETS:new(),
@@ -946,6 +1035,7 @@ terminate(_Reason, StateName, StateData) ->
 			    Packet = {xmlelement, "presence",
 				      [{"type", "unavailable"}], []},
 			    ejabberd_sm:unset_presence(StateData#state.user,
+						       StateData#state.server,
 						       StateData#state.resource,
 						       ""),
 			    presence_broadcast(
@@ -1036,6 +1126,7 @@ process_presence_probe(From, To, StateData) ->
 %-ifdef(PRIVACY_SUPPORT).
 		    case catch mod_privacy:check_packet(
 				 StateData#state.user,
+				 StateData#state.server,
 				 StateData#state.privacy_list,
 				 {To, From, Packet},
 				 out) of
@@ -1068,6 +1159,7 @@ presence_update(From, Packet, StateData) ->
 			    xml:get_tag_cdata(StatusTag)
 		     end,
 	    ejabberd_sm:unset_presence(StateData#state.user,
+				       StateData#state.server,
 				       StateData#state.resource,
 				       Status),
 	    presence_broadcast(StateData, From, StateData#state.pres_a, Packet),
@@ -1139,6 +1231,7 @@ presence_track(From, To, Packet, StateData) ->
     {xmlelement, _Name, Attrs, _Els} = Packet,
     LTo = jlib:jid_tolower(To),
     User = StateData#state.user,
+    Server = StateData#state.server,
     case xml:get_attr_s("type", Attrs) of
 	"unavailable" ->
 	    ejabberd_router:route(From, To, Packet),
@@ -1154,19 +1247,23 @@ presence_track(From, To, Packet, StateData) ->
 			    pres_a = A};
 	"subscribe" ->
 	    ejabberd_router:route(jlib:jid_remove_resource(From), To, Packet),
-	    mod_roster:out_subscription(User, To, subscribe),
+	    ejabberd_hooks:run(roster_out_subscription,
+			       [User, Server, To, subscribe]),
 	    StateData;
 	"subscribed" ->
 	    ejabberd_router:route(jlib:jid_remove_resource(From), To, Packet),
-	    mod_roster:out_subscription(User, To, subscribed),
+	    ejabberd_hooks:run(roster_out_subscription,
+			       [User, Server, To, subscribed]),
 	    StateData;
 	"unsubscribe" ->
 	    ejabberd_router:route(jlib:jid_remove_resource(From), To, Packet),
-	    mod_roster:out_subscription(User, To, unsubscribe),
+	    ejabberd_hooks:run(roster_out_subscription,
+			       [User, Server, To, unsubscribe]),
 	    StateData;
 	"unsubscribed" ->
 	    ejabberd_router:route(jlib:jid_remove_resource(From), To, Packet),
-	    mod_roster:out_subscription(User, To, unsubscribed),
+	    ejabberd_hooks:run(roster_out_subscription,
+			       [User, Server, To, unsubscribed]),
 	    StateData;
 	"error" ->
 	    ejabberd_router:route(From, To, Packet),
@@ -1178,6 +1275,7 @@ presence_track(From, To, Packet, StateData) ->
 %-ifdef(PRIVACY_SUPPORT).
 	    case catch mod_privacy:check_packet(
 			 StateData#state.user,
+			 StateData#state.server,
 			 StateData#state.privacy_list,
 			 {From, To, Packet},
 			 out) of
@@ -1201,6 +1299,7 @@ presence_broadcast(StateData, From, JIDSet, Packet) ->
 %-ifdef(PRIVACY_SUPPORT).
 			  case catch mod_privacy:check_packet(
 				       StateData#state.user,
+				       StateData#state.server,
 				       StateData#state.privacy_list,
 				       {From, FJID, Packet},
 				       out) of
@@ -1223,6 +1322,7 @@ presence_broadcast_to_trusted(StateData, From, T, A, Packet) ->
 %-ifdef(PRIVACY_SUPPORT).
 		      case catch mod_privacy:check_packet(
 				   StateData#state.user,
+				   StateData#state.server,
 				   StateData#state.privacy_list,
 				   {From, FJID, Packet},
 				   out) of
@@ -1262,6 +1362,7 @@ presence_broadcast_first(From, StateData, Packet) ->
 %-ifdef(PRIVACY_SUPPORT).
 			    case catch mod_privacy:check_packet(
 					 StateData#state.user,
+					 StateData#state.server,
 					 StateData#state.privacy_list,
 					 {From, FJID, Packet},
 					 out) of
@@ -1357,6 +1458,7 @@ update_priority(El, StateData) ->
 		  end
 	  end,
     ejabberd_sm:set_presence(StateData#state.user,
+			     StateData#state.server,
 			     StateData#state.resource,
 			     Pri).
 
@@ -1401,15 +1503,17 @@ process_privacy_iq(From, To,
 
 
 resend_offline_messages(#state{user = User,
+			       server = Server,
 			       privacy_list = PrivList} = StateData) ->
     case ejabberd_hooks:run_fold(resend_offline_messages_hook, [],
-				 [User]) of
+				 [User, Server]) of
 	Rs when list(Rs) ->
 	    lists:foreach(
 	      fun({route,
 		   From, To, {xmlelement, Name, Attrs, Els} = Packet}) ->
 		      Pass = case catch mod_privacy:check_packet(
 					  User,
+					  Server,
 					  PrivList,
 					  {From, To, Packet},
 					  in) of
