@@ -3,12 +3,12 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : 
 %%% Created : 14 Dec 2002 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: ejabberd_config.erl,v 1.11 2004/09/25 20:52:19 aleksey Exp $
+%%% Id      : $Id: ejabberd_config.erl 307 2005-04-17 18:08:34Z tmallard $
 %%%----------------------------------------------------------------------
 
 -module(ejabberd_config).
 -author('alexey@sevcom.net').
--vsn('$Revision: 1.11 $ ').
+-vsn('$Revision$ ').
 
 -export([start/0, load_file/1,
 	 add_global_option/2, add_local_option/2,
@@ -78,13 +78,15 @@ process_term(Term, State) ->
 	    State#state{opts = [#config{key = {shaper, Name},
 					value = Data} |
 				State#state.opts]};
+	{host, Host} ->
+	    add_option(hosts, [Host], State);
 	{Opt, Val} ->
 	    add_option(Opt, Val, State)
     end.
 
 add_option(Opt, Val, State) ->
     Table = case Opt of
-		host ->
+		hosts ->
 		    config;
 		language ->
 		    config;
