@@ -3,16 +3,19 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : Main router
 %%% Created : 27 Nov 2002 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: ejabberd_router.erl,v 1.21 2004/01/06 20:37:10 aleksey Exp $
+%%% Id      : $Id: ejabberd_router.erl 307 2005-04-17 18:08:34Z tmallard $
 %%%----------------------------------------------------------------------
 
 -module(ejabberd_router).
 -author('alexey@sevcom.net').
--vsn('$Revision: 1.21 $ ').
+-vsn('$Revision$ ').
 
 -export([route/3,
 	 register_route/1,
+	 register_route/2,
+	 register_routes/1,
 	 unregister_route/1,
+	 unregister_routes/1,
 	 dirty_get_all_routes/0,
 	 dirty_get_all_domains/0
 	]).
@@ -22,15 +25,10 @@
 -include("ejabberd.hrl").
 -include("jlib.hrl").
 
--record(route, {domain, pid}).
+-record(route, {domain, pid, local_hint}).
 
 
 start_link() ->
-    Pid = proc_lib:spawn_link(ejabberd_router, init, []),
-    register(ejabberd_router, Pid),
-    {ok, Pid}.
-
-init() ->
     update_tables(),
     mnesia:create_table(route,
 			[{ram_copies, [node()]},
@@ -38,7 +36,17 @@ init() ->
 			 {attributes,
 			  record_info(fields, route)}]),
     mnesia:add_table_copy(route, node(), ram_copies),
+    Pid = proc_lib:spawn_link(ejabberd_router, init, []),
+    register(ejabberd_router, Pid),
+    {ok, Pid}.
+
+init() ->
     mnesia:subscribe({table, route, simple}),
+    lists:foreach(
+      fun(Pid) ->
+	      erlang:monitor(process, Pid)
+      end,
+      mnesia:dirty_select(route, [{{route, '_', '$1', '_'}, [], ['$1']}])),
     loop().
 
 loop() ->
@@ -52,20 +60,6 @@ loop() ->
 		    ok
 	    end,
 	    loop();
-	{register_route, Domain, Pid} ->
-	    F = fun() ->
-			mnesia:write(#route{domain = Domain,
-					    pid = Pid})
-		end,
-	    mnesia:transaction(F),
-	    loop();
-	{unregister_route, Domain, Pid} ->
-	    F = fun() ->
-			mnesia:delete_object(#route{domain = Domain,
-						    pid = Pid})
-		end,
-	    mnesia:transaction(F),
-	    loop();
 	{mnesia_table_event, {write, #route{pid = Pid}, _ActivityId}} ->
 	    erlang:monitor(process, Pid),
 	    loop();
@@ -91,35 +85,105 @@ do_route(From, To, Packet) ->
     LDstDomain = To#jid.lserver,
     case mnesia:dirty_read(route, LDstDomain) of
 	[] ->
-	    ejabberd_s2s ! {route, From, To, Packet};
+	    ejabberd_s2s:route(From, To, Packet);
 	[R] ->
 	    Pid = R#route.pid,
-	    ?DEBUG("routed to process ~p~n", [Pid]),
-	    Pid ! {route, From, To, Packet};
+	    if
+		node(Pid) == node() ->
+		    case R#route.local_hint of
+			{apply, Module, Function} ->
+			    Module:Function(From, To, Packet);
+			_ ->
+			    Pid ! {route, From, To, Packet}
+		    end;
+		true ->
+		    Pid ! {route, From, To, Packet}
+	    end;
 	Rs ->
-	    Rs1 = case [R || R <- Rs, node(R#route.pid) == node()] of
-		      [] -> Rs;
-		      LRs -> LRs
-		  end,
-	    R = lists:nth(erlang:phash(now(), length(Rs1)), Rs1),
-	    Pid = R#route.pid,
-	    ?DEBUG("routed to process ~p~n", [Pid]),
-	    Pid ! {route, From, To, Packet}
+	    case [R || R <- Rs, node(R#route.pid) == node()] of
+		[] ->
+		    R = lists:nth(erlang:phash(now(), length(Rs)), Rs),
+		    Pid = R#route.pid,
+		    Pid ! {route, From, To, Packet};
+		LRs ->
+		    LRs,
+		    R = lists:nth(erlang:phash(now(), length(LRs)), LRs),
+		    Pid = R#route.pid,
+		    case R#route.local_hint of
+			{apply, Module, Function} ->
+			    Module:Function(From, To, Packet);
+			_ ->
+			    Pid ! {route, From, To, Packet}
+		    end
+	    end
     end.
 
 
+%route(From, To, Packet) ->
+%    ejabberd_router ! {route, From, To, Packet}.
+
 route(From, To, Packet) ->
-    ejabberd_router ! {route, From, To, Packet}.
+    case catch do_route(From, To, Packet) of
+	{'EXIT', Reason} ->
+	    ?ERROR_MSG("~p~nwhen processing: ~p",
+		       [Reason, {From, To, Packet}]);
+	_ ->
+	    ok
+    end.
 
 register_route(Domain) ->
-    ejabberd_router ! {register_route, Domain, self()}.
+    case jlib:nameprep(Domain) of
+	error ->
+	    [] = {invalid_domain, Domain};
+	LDomain ->
+	    Pid = self(),
+	    F = fun() ->
+			mnesia:write(#route{domain = LDomain,
+					    pid = Pid})
+		end,
+	    mnesia:transaction(F)
+    end.
+
+register_route(Domain, LocalHint) ->
+    case jlib:nameprep(Domain) of
+	error ->
+	    [] = {invalid_domain, Domain};
+	LDomain ->
+	    Pid = self(),
+	    F = fun() ->
+			mnesia:write(#route{domain = LDomain,
+					    pid = Pid,
+					    local_hint = LocalHint})
+		end,
+	    mnesia:transaction(F)
+    end.
+
+register_routes(Domains) ->
+    lists:foreach(fun(Domain) ->
+			  register_route(Domain)
+		  end, Domains).
 
 unregister_route(Domain) ->
-    ejabberd_router ! {unregister_route, Domain, self()}.
+    case jlib:nameprep(Domain) of
+	error ->
+	    [] = {invalid_domain, Domain};
+	LDomain ->
+	    Pid = self(),
+	    F = fun() ->
+			mnesia:delete_object(#route{domain = LDomain,
+						    pid = Pid})
+		end,
+	    mnesia:transaction(F)
+    end.
+
+unregister_routes(Domains) ->
+    lists:foreach(fun(Domain) ->
+			  unregister_route(Domain)
+		  end, Domains).
 
 
 dirty_get_all_routes() ->
-    lists:delete(?MYNAME, lists:usort(mnesia:dirty_all_keys(route))).
+    lists:usort(mnesia:dirty_all_keys(route)) -- ?MYHOSTS.
 
 dirty_get_all_domains() ->
     lists:usort(mnesia:dirty_all_keys(route)).
@@ -131,6 +195,8 @@ update_tables() ->
 	[domain, node, pid] ->
 	    mnesia:delete_table(route);
 	[domain, pid] ->
+	    mnesia:delete_table(route);
+	[domain, pid, local_hint] ->
 	    ok;
 	{'EXIT', _} ->
 	    ok
