@@ -3,20 +3,24 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : Session manager
 %%% Created : 24 Nov 2002 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: ejabberd_sm.erl,v 1.43 2004/10/08 20:40:28 aleksey Exp $
+%%% Id      : $Id: ejabberd_sm.erl 309 2005-04-17 21:39:41Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(ejabberd_sm).
 -author('alexey@sevcom.net').
--vsn('$Revision: 1.43 $ ').
+-vsn('$Revision$ ').
 
--export([start_link/0, init/0, open_session/2, close_session/2,
+-export([start_link/0, init/0,
+	 route/3,
+	 open_session/3, close_session/3,
 	 bounce_offline_message/3,
-	 get_user_resources/1,
-	 set_presence/3,
-	 unset_presence/3,
+	 disconnect_removed_user/2,
+	 get_user_resources/2,
+	 set_presence/4,
+	 unset_presence/4,
 	 dirty_get_sessions_list/0,
 	 dirty_get_my_sessions_list/0,
+	 get_vh_session_list/1,
 	 register_iq_handler/3,
 	 register_iq_handler/4,
 	 unregister_iq_handler/1
@@ -25,8 +29,8 @@
 -include("ejabberd.hrl").
 -include("jlib.hrl").
 
--record(session, {ur, user, pid}).
--record(presence, {ur, user, priority}).
+-record(session, {usr, us, pid}).
+-record(presence, {usr, us, priority}).
 
 start_link() ->
     Pid = proc_lib:spawn_link(ejabberd_sm, init, []),
@@ -37,16 +41,18 @@ init() ->
     update_tables(),
     mnesia:create_table(session, [{ram_copies, [node()]},
 				  {attributes, record_info(fields, session)}]),
-    mnesia:add_table_index(session, user),
+    mnesia:add_table_index(session, us),
     mnesia:add_table_copy(session, node(), ram_copies),
     mnesia:create_table(presence,
 			[{ram_copies, [node()]},
 			 {attributes, record_info(fields, presence)}]),
-    mnesia:add_table_index(presence, user),
+    mnesia:add_table_index(presence, us),
     mnesia:subscribe(system),
     ets:new(sm_iqtable, [named_table]),
     ejabberd_hooks:add(offline_message_hook,
 		       ejabberd_sm, bounce_offline_message, 100),
+    ejabberd_hooks:add(remove_user,
+		       ejabberd_sm, disconnect_removed_user, 100),
     loop().
 
 loop() ->
@@ -60,12 +66,6 @@ loop() ->
 		    ok
 	    end,
 	    loop();
-	{open_session, User, Resource, From} ->
-	    register_connection(User, Resource, From),
-	    loop();
-	{close_session, User, Resource} ->
-	    remove_connection(User, Resource),
-	    loop();
 	{mnesia_system_event, {mnesia_down, Node}} ->
 	    clean_table_from_bad_node(Node),
 	    loop();
@@ -89,20 +89,31 @@ loop() ->
     end.
 
 
-open_session(User, Resource) ->
-    ejabberd_sm ! {open_session, User, Resource, self()}.
+route(From, To, Packet) ->
+    case catch do_route(From, To, Packet) of
+	{'EXIT', Reason} ->
+	    ?ERROR_MSG("~p~nwhen processing: ~p",
+		       [Reason, {From, To, Packet}]);
+	_ ->
+	    ok
+    end.
 
-close_session(User, Resource) ->
-    ejabberd_sm ! {close_session, User, Resource}.
+open_session(User, Server, Resource) ->
+    register_connection(User, Server, Resource, self()).
+
+close_session(User, Server, Resource) ->
+    remove_connection(User, Server, Resource).
 
 
-register_connection(User, Resource, Pid) ->
+register_connection(User, Server, Resource, Pid) ->
     LUser = jlib:nodeprep(User),
+    LServer = jlib:nameprep(Server),
     LResource = jlib:resourceprep(Resource),
-    UR = {LUser, LResource},
+    US = {LUser, LServer},
+    USR = {LUser, LServer, LResource},
     F = fun() ->
-		Ss = mnesia:wread({session, UR}),
-		mnesia:write(#session{ur = UR, user = LUser, pid = Pid}),
+		Ss = mnesia:wread({session, USR}),
+		mnesia:write(#session{usr = USR, us = US, pid = Pid}),
 		Ss
         end,
     case mnesia:transaction(F) of
@@ -116,12 +127,13 @@ register_connection(User, Resource, Pid) ->
     end.
 
 
-remove_connection(User, Resource) ->
+remove_connection(User, Server, Resource) ->
     LUser = jlib:nodeprep(User),
     LResource = jlib:resourceprep(Resource),
-    UR = {LUser, LResource},
+    LServer = jlib:nameprep(Server),
+    USR = {LUser, LServer, LResource},
     F = fun() ->
-		mnesia:delete({session, UR})
+		mnesia:delete({session, USR})
         end,
     mnesia:transaction(F).
 
@@ -134,7 +146,8 @@ clean_table_from_bad_node(Node) ->
 			 [{'==', {node, '$1'}, Node}],
 			 ['$_']}]),
 		lists:foreach(fun(E) ->
-				      mnesia:delete_object(E)
+				      mnesia:delete_object(E),
+				      mnesia:delete({presence, E#session.usr})
 			      end, Es)
         end,
     mnesia:transaction(F).
@@ -145,7 +158,7 @@ clean_table_from_bad_node(Node) ->
 do_route(From, To, Packet) ->
     ?DEBUG("session manager~n\tfrom ~p~n\tto ~p~n\tpacket ~P~n",
 	   [From, To, Packet, 8]),
-    #jid{user = User,
+    #jid{user = User, server = Server,
 	 luser = LUser, lserver = LServer, lresource = LResource} = To,
     {xmlelement, Name, Attrs, _Els} = Packet,
     case LResource of
@@ -155,42 +168,46 @@ do_route(From, To, Packet) ->
 		    {Pass, Subsc} =
 			case xml:get_attr_s("type", Attrs) of
 			    "subscribe" ->
-				{mod_roster:in_subscription(User,
-							    From,
-							    subscribe),
+				{ejabberd_hooks:run_fold(
+				   roster_in_subscription,
+				   false,
+				   [User, Server, From, subscribe]),
 				 true};
 			    "subscribed" ->
-				{mod_roster:in_subscription(User,
-							    From,
-							    subscribed),
+				{ejabberd_hooks:run_fold(
+				   roster_in_subscription,
+				   false,
+				   [User, Server, From, subscribed]),
 				 true};
 			    "unsubscribe" ->
-				{mod_roster:in_subscription(User,
-							    From,
-							    unsubscribe),
+				{ejabberd_hooks:run_fold(
+				   roster_in_subscription,
+				   false,
+				   [User, Server, From, unsubscribe]),
 				 true};
 			    "unsubscribed" ->
-				{mod_roster:in_subscription(User,
-							    From,
-							    unsubscribed),
+				{ejabberd_hooks:run_fold(
+				   roster_in_subscription,
+				   false,
+				   [User, Server, From, unsubscribed]),
 				 true};
 			    _ ->
 				{true, false}
 			end,
 		    if Pass ->
 			    LFrom = jlib:jid_tolower(From),
-			    PResources = get_user_present_resources(User),
+			    PResources = get_user_present_resources(
+					   LUser, LServer),
 			    if
 				PResources /= [] ->
 				    lists:foreach(
 				      fun({_, R}) ->
 					      if LFrom /=
 						 {LUser, LServer, R} ->
-						      ejabberd_sm !
-							  {route,
-							   From,
-							   jlib:jid_replace_resource(To, R),
-							   Packet};
+						      do_route(
+							From,
+							jlib:jid_replace_resource(To, R),
+							Packet);
 						 true ->
 						      ok
 					      end
@@ -198,7 +215,8 @@ do_route(From, To, Packet) ->
 				true ->
 				    if
 					Subsc ->
-					    case ejabberd_auth:is_user_exists(LUser) of
+					    case ejabberd_auth:is_user_exists(
+						   LUser, LServer) of
 						true ->
 						    ejabberd_hooks:run(
 						      offline_subscription_hook,
@@ -222,17 +240,16 @@ do_route(From, To, Packet) ->
 		"broadcast" ->
 		    lists:foreach(
 		      fun(R) ->
-			      ejabberd_sm ! {route,
-					     From,
-					     jlib:jid_replace_resource(To, R),
-					     Packet}
-		      end, get_user_resources(User));
+			      do_route(From,
+				       jlib:jid_replace_resource(To, R),
+				       Packet)
+		      end, get_user_resources(User, Server));
 		_ ->
 		    ok
 	    end;
 	_ ->
-	    LUR = {LUser, LResource},
-	    case mnesia:dirty_read({session, LUR}) of
+	    USR = {LUser, LServer, LResource},
+	    case mnesia:dirty_read({session, USR}) of
 		[] ->
 		    case Name of
 			"message" ->
@@ -259,12 +276,13 @@ do_route(From, To, Packet) ->
 
 route_message(From, To, Packet) ->
     LUser = To#jid.luser,
-    case catch lists:max(get_user_present_resources(LUser)) of
+    LServer = To#jid.lserver,
+    case catch lists:max(get_user_present_resources(LUser, LServer)) of
 	{Priority, R} when is_integer(Priority),
 			   Priority >= 0 ->
 	    LResource = jlib:resourceprep(R),
-	    LUR = {LUser, LResource},
-	    case mnesia:dirty_read({session, LUR}) of
+	    USR = {LUser, LServer, LResource},
+	    case mnesia:dirty_read({session, USR}) of
 		[] ->
 		    ok;				% Race condition
 		[Sess] ->
@@ -277,7 +295,7 @@ route_message(From, To, Packet) ->
 		"error" ->
 		    ok;
 		_ ->
-		    case ejabberd_auth:is_user_exists(LUser) of
+		    case ejabberd_auth:is_user_exists(LUser, LServer) of
 			true ->
 			    ejabberd_hooks:run(offline_message_hook,
 					       [From, To, Packet]);
@@ -294,47 +312,58 @@ bounce_offline_message(From, To, Packet) ->
     ejabberd_router:route(To, From, Err),
     stop.
 
+disconnect_removed_user(User, Server) ->
+    ejabberd_sm:route(jlib:make_jid("", "", ""),
+		      jlib:make_jid(User, Server, ""),
+		      {xmlelement, "broadcast", [],
+		       [{exit, "User removed"}]}).
+
+
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-get_user_resources(User) ->
+get_user_resources(User, Server) ->
     LUser = jlib:nodeprep(User),
-    case catch mnesia:dirty_index_read(session, LUser, #session.user) of
+    LServer = jlib:nameprep(Server),
+    US = {LUser, LServer},
+    case catch mnesia:dirty_index_read(session, US, #session.us) of
 	{'EXIT', _Reason} ->
 	    [];
 	Rs ->
 	    lists:map(fun(R) ->
-			      element(2, R#session.ur)
+			      element(3, R#session.usr)
 		      end, Rs)
     end.
 
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-set_presence(User, Resource, Priority) ->
+set_presence(User, Server, Resource, Priority) ->
     LUser = jlib:nodeprep(User),
+    LServer = jlib:nameprep(Server),
+    USR = {User, Server, Resource},
+    US = {LUser, LServer},
     F = fun() ->
-		UR = {User, Resource},
-		mnesia:write(#presence{ur = UR, user = LUser,
+		mnesia:write(#presence{usr = USR, us = US,
 				       priority = Priority})
 	end,
     mnesia:transaction(F).
 
-unset_presence(User, Resource, Status) ->
-    LUser = jlib:nodeprep(User),
+unset_presence(User, Server, Resource, Status) ->
+    USR = {User, Server, Resource},
     F = fun() ->
-		UR = {User, Resource},
-		mnesia:delete({presence, UR})
+		mnesia:delete({presence, USR})
 	end,
     mnesia:transaction(F),
-    catch mod_last:on_presence_update(LUser, Status).
+    ejabberd_hooks:run(unset_presence_hook, [User, Server, Resource, Status]).
 
-get_user_present_resources(LUser) ->
-    case catch mnesia:dirty_index_read(presence, LUser, #presence.user) of
+get_user_present_resources(LUser, LServer) ->
+    US = {LUser, LServer},
+    case catch mnesia:dirty_index_read(presence, US, #presence.us) of
 	{'EXIT', _Reason} ->
 	    [];
 	Rs ->
 	    lists:map(fun(R) ->
-			      {R#presence.priority, element(2, R#presence.ur)}
+			      {R#presence.priority, element(3, R#presence.usr)}
 		      end, Rs)
     end.
 
@@ -348,6 +377,14 @@ dirty_get_my_sessions_list() ->
 	[{'==', {node, '$1'}, node()}],
 	['$_']}]).
 
+get_vh_session_list(Server) ->
+    LServer = jlib:nameprep(Server),
+    mnesia:dirty_select(
+      session,
+      [{#session{usr = '$1', _ = '_'},
+	[{'==', {element, 2, '$1'}, LServer}],
+	['$1']}]).
+
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
@@ -360,10 +397,8 @@ process_iq(From, To, Packet) ->
 		    ResIQ = Module:Function(From, To, IQ),
 		    if
 			ResIQ /= ignore ->
-			    ejabberd_router ! {route,
-					       To,
-					       From,
-					       jlib:iq_to_xml(ResIQ)};
+			    ejabberd_router:route(To, From,
+						  jlib:iq_to_xml(ResIQ));
 			true ->
 			    ok
 		    end;
@@ -373,7 +408,7 @@ process_iq(From, To, Packet) ->
 		[] ->
 		    Err = jlib:make_error_reply(
 			    Packet, ?ERR_FEATURE_NOT_IMPLEMENTED),
-		    ejabberd_router ! {route, To, From, Err}
+		    ejabberd_router:route(To, From, Err)
 	    end;
 	reply ->
 	    ok;
@@ -399,6 +434,16 @@ update_tables() ->
 	[ur, user, node] ->
 	    mnesia:delete_table(session);
 	[ur, user, pid] ->
+	    mnesia:delete_table(session);
+	[usr, us, pid] ->
+	    ok;
+	{'EXIT', _} ->
+	    ok
+    end,
+    case catch mnesia:table_info(presence, attributes) of
+	[ur, user, priority] ->
+	    mnesia:delete_table(presence);
+	[usr, us, priority] ->
 	    ok;
 	{'EXIT', _} ->
 	    ok
