@@ -3,12 +3,12 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : 
 %%% Created : 15 Jan 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: mod_echo.erl,v 1.5 2003/12/14 20:50:59 aleksey Exp $
+%%% Id      : $Id: mod_echo.erl 354 2005-05-23 00:30:29Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(mod_echo).
 -author('alexey@sevcom.net').
--vsn('$Revision: 1.5 $ ').
+-vsn('$Revision: 354 $ ').
 
 -behaviour(gen_mod).
 
@@ -17,12 +17,12 @@
 -include("ejabberd.hrl").
 -include("jlib.hrl").
 
-
+-define(PROCNAME, ejabberd_mod_echo).
 
 start(Opts) ->
     %Host = gen_mod:get_opt(host, Opts),
     Host = gen_mod:get_opt(host, Opts, "echo." ++ ?MYNAME),
-    register(ejabberd_mod_echo, spawn(?MODULE, init, [Host])).
+    register(?PROCNAME, spawn(?MODULE, init, [Host])).
 
 init(Host) ->
     ejabberd_router:register_route(Host),
@@ -41,5 +41,6 @@ loop(Host) ->
     end.
 
 stop() ->
-    ejabberd_mod_echo ! stop,
-    ok.
+    ?PROCNAME ! stop,
+    {wait, ?PROCNAME}.
+
