@@ -3,12 +3,12 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : jabber:iq:last support (JEP-0012)
 %%% Created : 24 Oct 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: mod_last.erl,v 1.5 2004/07/11 20:51:50 aleksey Exp $
+%%% Id      : $Id: mod_last.erl 358 2005-05-23 19:47:57Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(mod_last).
 -author('alexey@sevcom.net').
--vsn('$Revision: 1.5 $ ').
+-vsn('$Revision: 358 $ ').
 
 -behaviour(gen_mod).
 
@@ -16,13 +16,14 @@
 	 stop/0,
 	 process_local_iq/3,
 	 process_sm_iq/3,
-	 on_presence_update/2,
-	 remove_user/1]).
+	 on_presence_update/4,
+	 store_last_info/4,
+	 remove_user/2]).
 
 -include("ejabberd.hrl").
 -include("jlib.hrl").
 
--record(last_activity, {user, timestamp, status}).
+-record(last_activity, {us, timestamp, status}).
 
 
 start(Opts) ->
@@ -34,9 +35,17 @@ start(Opts) ->
     gen_iq_handler:add_iq_handler(ejabberd_local, ?NS_LAST,
 				  ?MODULE, process_local_iq, IQDisc),
     gen_iq_handler:add_iq_handler(ejabberd_sm, ?NS_LAST,
-				  ?MODULE, process_sm_iq, IQDisc).
+				  ?MODULE, process_sm_iq, IQDisc),
+    ejabberd_hooks:add(remove_user,
+		       ?MODULE, remove_user, 50),
+    ejabberd_hooks:add(unset_presence_hook,
+		       ?MODULE, on_presence_update, 50).
 
 stop() ->
+    ejabberd_hooks:delete(remove_user,
+			  ?MODULE, remove_user, 50),
+    ejabberd_hooks:delete(unset_presence_hook,
+			  ?MODULE, on_presence_update, 50),
     gen_iq_handler:remove_iq_handler(ejabberd_local, ?NS_LAST),
     gen_iq_handler:remove_iq_handler(ejabberd_sm, ?NS_LAST).
 
@@ -60,23 +69,25 @@ process_sm_iq(From, To, #iq{type = Type, sub_el = SubEl} = IQ) ->
 	    IQ#iq{type = error, sub_el = [SubEl, ?ERR_NOT_ALLOWED]};
 	get ->
 	    User = To#jid.luser,
+	    Server = To#jid.lserver,
 	    {Subscription, _Groups} =
-		mod_roster:get_jid_info(User, From),
+		ejabberd_hooks:run_fold(
+		  roster_get_jid_info, {none, []}, [User, Server, From]),
 	    if
 		(Subscription == both) or (Subscription == from) ->
-		    case catch mod_privacy:get_user_list(User) of
+		    case catch mod_privacy:get_user_list(User, Server) of
 			{'EXIT', _Reason} ->
-			    get_last(IQ, SubEl, User);
+			    get_last(IQ, SubEl, User, Server);
 			List ->
 			    case catch mod_privacy:check_packet(
-					 User, List,
+					 User, Server, List,
 					 {From, To,
 					  {xmlelement, "presence", [], []}},
 					 out) of
 				{'EXIT', _Reason} ->
-				    get_last(IQ, SubEl, User);
+				    get_last(IQ, SubEl, User, Server);
 				allow ->
-				    get_last(IQ, SubEl, User);
+				    get_last(IQ, SubEl, User, Server);
 				deny ->
 				    IQ#iq{type = error,
 					  sub_el = [SubEl, ?ERR_NOT_ALLOWED]}
@@ -88,8 +99,8 @@ process_sm_iq(From, To, #iq{type = Type, sub_el = SubEl} = IQ) ->
 	    end
     end.
 
-get_last(IQ, SubEl, LUser) ->
-    case catch mnesia:dirty_read(last_activity, LUser) of
+get_last(IQ, SubEl, LUser, LServer) ->
+    case catch mnesia:dirty_read(last_activity, {LUser, LServer}) of
 	{'EXIT', _Reason} ->
 	    IQ#iq{type = error, sub_el = [SubEl, ?ERR_INTERNAL_SERVER_ERROR]};
 	[] ->
@@ -107,21 +118,29 @@ get_last(IQ, SubEl, LUser) ->
 
 
 
-on_presence_update(LUser, Status) ->
+on_presence_update(User, Server, _Resource, Status) ->
     {MegaSecs, Secs, _MicroSecs} = now(),
     TimeStamp = MegaSecs * 1000000 + Secs,
+    store_last_info(User, Server, TimeStamp, Status).
+
+store_last_info(User, Server, TimeStamp, Status) ->
+    LUser = jlib:nodeprep(User),
+    LServer = jlib:nameprep(Server),
+    US = {LUser, LServer},
     F = fun() ->
-		mnesia:write(#last_activity{user = LUser,
+		mnesia:write(#last_activity{us = US,
 					    timestamp = TimeStamp,
 					    status = Status})
 	end,
     mnesia:transaction(F).
+    
 
-
-remove_user(User) ->
+remove_user(User, Server) ->
     LUser = jlib:nodeprep(User),
+    LServer = jlib:nameprep(Server),
+    US = {LUser, LServer},
     F = fun() ->
-		mnesia:delete({last_activity, LUser})
+		mnesia:delete({last_activity, US})
 	end,
     mnesia:transaction(F).
 
@@ -131,13 +150,44 @@ update_table() ->
     case mnesia:table_info(last_activity, attributes) of
 	Fields ->
 	    ok;
+	[user, timestamp, status] ->
+	    ?INFO_MSG("Converting last_activity table from {user, timestamp, status} format", []),
+	    Host = ?MYNAME,
+	    mnesia:transform_table(last_activity, ignore, Fields),
+	    F = fun() ->
+			mnesia:write_lock_table(last_activity),
+			mnesia:foldl(
+			  fun({_, U, T, S} = R, _) ->
+				  mnesia:delete_object(R),
+				  mnesia:write(
+				    #last_activity{us = {U, Host},
+						   timestamp = T,
+						   status = S})
+			  end, ok, last_activity)
+		end,
+	    mnesia:transaction(F);
 	[user, timestamp] ->
 	    ?INFO_MSG("Converting last_activity table from {user, timestamp} format", []),
+	    Host = ?MYNAME,
 	    mnesia:transform_table(
 	      last_activity,
 	      fun({_, U, T}) ->
-		      #last_activity{user = U, timestamp = T, status = ""}
-	      end, Fields);
+		      #last_activity{us = U,
+				     timestamp = T,
+				     status = ""}
+	      end, Fields),
+	    F = fun() ->
+			mnesia:write_lock_table(last_activity),
+			mnesia:foldl(
+			  fun({_, U, T, S} = R, _) ->
+				  mnesia:delete_object(R),
+				  mnesia:write(
+				    #last_activity{us = {U, Host},
+						   timestamp = T,
+						   status = S})
+			  end, ok, last_activity)
+		end,
+	    mnesia:transaction(F);
 	_ ->
 	    ?INFO_MSG("Recreating last_activity table", []),
 	    mnesia:transform_table(last_activity, ignore, Fields)
