@@ -3,12 +3,12 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : MUC room stuff
 %%% Created : 19 Mar 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: mod_muc_room.erl,v 1.31 2004/10/08 20:40:28 aleksey Exp $
+%%% Id      : $Id: mod_muc_room.erl 346 2005-05-09 19:38:16Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(mod_muc_room).
 -author('alexey@sevcom.net').
--vsn('$Revision: 1.31 $ ').
+-vsn('$Revision: 346 $ ').
 
 -behaviour(gen_fsm).
 
@@ -149,6 +149,7 @@ normal_state({route, From, "",
 						case (NSD#state.config)#config.persistent of
 						    true ->
 							mod_muc:store_room(
+							  NSD#state.host,
 							  NSD#state.room,
 							  make_opts(NSD));
 						    _ ->
@@ -177,20 +178,23 @@ normal_state({route, From, "",
 							       NewStateData1),
 				    {next_state, normal_state, NewStateData2};
 				_ ->
-				    ErrText = 
+				    Err = 
 					case (StateData#state.config)#config.allow_change_subj of
 					    true ->
-						"Only moderators and participants "
-						"are allowed to change subject in this room";
+						?ERRT_FORBIDDEN(
+						  Lang,
+						  "Only moderators and participants "
+						  "are allowed to change subject in this room");
 					    _ ->
-						"Only moderators "
-						"are allowed to change subject in this room"
+						?ERRT_FORBIDDEN(
+						  Lang,
+						  "Only moderators "
+						  "are allowed to change subject in this room")
 					end,
-				    Err = jlib:make_error_reply(
-					    Packet, ?ERRT_FORBIDDEN(Lang, ErrText)),
 				    ejabberd_router:route(
 				      StateData#state.jid,
-				      From, Err),
+				      From,
+				      jlib:make_error_reply(Packet, Err)),
 			    {next_state, normal_state, StateData}
 			    end;
 			true ->
@@ -245,6 +249,15 @@ normal_state({route, From, "",
 						    IJID,
 						    member,
 						    StateData),
+					    case (NSD#state.config)#config.persistent of
+						true ->
+						    mod_muc:store_room(
+						      NSD#state.host,
+						      NSD#state.room,
+						      make_opts(NSD));
+						_ ->
+						    ok
+					    end,
 					    {next_state, normal_state, NSD};
 					_ ->
 					    {next_state, normal_state,
@@ -364,7 +377,8 @@ normal_state({route, From, Nick,
 			case is_nick_change(From, Nick, StateData) of
 			    true ->
 				case {is_nick_exists(Nick, StateData),
-				      mod_muc:can_use_nick(From, Nick)} of
+				      mod_muc:can_use_nick(
+					StateData#state.host, From, Nick)} of
 				    {true, _} ->
 					Lang = xml:get_attr_s("xml:lang", Attrs),
 					ErrText = "Nickname is already in use by another occupant",
@@ -542,7 +556,6 @@ normal_state({route, From, ToNick,
     {next_state, normal_state, StateData};
 
 normal_state(Event, StateData) ->
-    io:format("MUC: unknown event ~p~n", [Event]),
     {next_state, normal_state, StateData}.
 
 
@@ -639,7 +652,7 @@ handle_info(_Info, StateName, StateData) ->
 %% Returns: any
 %%----------------------------------------------------------------------
 terminate(_Reason, _StateName, StateData) ->
-    mod_muc:room_destroyed(StateData#state.room),
+    mod_muc:room_destroyed(StateData#state.host, StateData#state.room),
     ok.
 
 %%%----------------------------------------------------------------------
@@ -703,19 +716,57 @@ set_affiliation(JID, Affiliation, StateData) ->
 		   end,
     StateData#state{affiliations = Affiliations}.
 
+set_affiliation_and_reason(JID, Affiliation, Reason, StateData) ->
+    LJID = jlib:jid_remove_resource(jlib:jid_tolower(JID)),
+    Affiliations = case Affiliation of
+		       none ->
+			   ?DICT:erase(LJID,
+				       StateData#state.affiliations);
+		       _ ->
+			   ?DICT:store(LJID,
+				       {Affiliation, Reason},
+				       StateData#state.affiliations)
+		   end,
+    StateData#state{affiliations = Affiliations}.
+
 get_affiliation(JID, StateData) ->
     {_AccessRoute, _AccessCreate, AccessAdmin} = StateData#state.access,
-    case acl:match_rule(AccessAdmin, JID) of
-	allow ->
-	    owner;
+    Res =
+	case acl:match_rule(AccessAdmin, JID) of
+	    allow ->
+		owner;
+	    _ ->
+		LJID = jlib:jid_tolower(JID),
+		case ?DICT:find(LJID, StateData#state.affiliations) of
+		    {ok, Affiliation} ->
+			Affiliation;
+		    _ ->
+			LJID1 = jlib:jid_remove_resource(LJID),
+			case ?DICT:find(LJID1, StateData#state.affiliations) of
+			    {ok, Affiliation} ->
+				Affiliation;
+			    _ ->
+				LJID2 = setelement(1, LJID, ""),
+				case ?DICT:find(LJID2, StateData#state.affiliations) of
+				    {ok, Affiliation} ->
+					Affiliation;
+				    _ ->
+					LJID3 = jlib:jid_remove_resource(LJID2),
+					case ?DICT:find(LJID3, StateData#state.affiliations) of
+					    {ok, Affiliation} ->
+						Affiliation;
+					    _ ->
+						none
+					end
+				end
+			end
+		end
+	end,
+    case Res of
+	{A, _Reason} ->
+	    A;
 	_ ->
-	    LJID = jlib:jid_remove_resource(jlib:jid_tolower(JID)),
-	    case ?DICT:find(LJID, StateData#state.affiliations) of
-		{ok, Affiliation} ->
-		    Affiliation;
-		_ ->
-		    none
-	    end
+	    Res
     end.
 
 set_role(JID, Role, StateData) ->
@@ -806,8 +857,8 @@ filter_presence({xmlelement, "presence", Attrs, Els}) ->
 		     case El of
 			 {xmlcdata, _} ->
 			     false;
-			 {xmlelement, Name1, _Attrs1, _Els1} ->
-			     XMLNS = xml:get_attr_s("xmlns", Attrs),
+			 {xmlelement, Name1, Attrs1, _Els1} ->
+			     XMLNS = xml:get_attr_s("xmlns", Attrs1),
 			     case {Name1, XMLNS} of
 				 {"show", ""} ->
 				     true;
@@ -872,7 +923,7 @@ is_nick_change(JID, Nick, StateData) ->
 add_new_user(From, Nick, {xmlelement, _, Attrs, Els} = Packet, StateData) ->
     Lang = xml:get_attr_s("xml:lang", Attrs),
     case {is_nick_exists(Nick, StateData),
-	  mod_muc:can_use_nick(From, Nick)} of
+	  mod_muc:can_use_nick(StateData#state.host, From, Nick)} of
 	{true, _} ->
 	    ErrText = "Nickname is already in use by another occupant",
 	    Err = jlib:make_error_reply(Packet, ?ERRT_CONFLICT(Lang, ErrText)),
@@ -1023,7 +1074,6 @@ count_stanza_shift(Nick, Els, StateData) ->
 		 _ ->
 		     count_maxchars_shift(Nick, MaxChars, HL)
 	     end,
-   
     lists:max([Shift0, Shift1, Shift2, Shift3]).
 
 count_seconds_shift(Seconds, HistoryList) ->
@@ -1087,7 +1137,7 @@ extract_history([{xmlelement, _Name, Attrs, _SubEls} = El | Els], Type) ->
 		    end;
 		_ ->
 		    case catch list_to_integer(AttrVal) of
-			IntVal when is_integer(IntVal) and IntVal >= 0 ->
+			IntVal when is_integer(IntVal) and (IntVal >= 0) ->
 			    IntVal;
 			_ ->
 			    false
@@ -1433,7 +1483,12 @@ items_with_role(SRole, StateData) ->
 
 items_with_affiliation(SAffiliation, StateData) ->
     lists:map(
-      fun({JID, Affiliation}) ->
+      fun({JID, {Affiliation, Reason}}) ->
+	      {xmlelement, "item",
+	       [{"affiliation", affiliation_to_list(Affiliation)},
+		{"jid", jlib:jid_to_string(JID)}],
+	       [{xmlelement, "reason", [], [{xmlcdata, Reason}]}]};
+	 ({JID, Affiliation}) ->
 	      {xmlelement, "item",
 	       [{"affiliation", affiliation_to_list(Affiliation)},
 		{"jid", jlib:jid_to_string(JID)}],
@@ -1461,7 +1516,12 @@ search_role(Role, StateData) ->
 search_affiliation(Affiliation, StateData) ->
     lists:filter(
       fun({_, A}) ->
-	      Affiliation == A
+	      case A of
+		  {A1, _Reason} ->
+		      Affiliation == A1;
+		  _ ->
+		      Affiliation == A
+	      end
       end, ?DICT:to_list(StateData#state.affiliations)).
 
 
@@ -1482,11 +1542,11 @@ process_admin_items_set(UJID, Items, Lang, StateData) ->
 				     {JID, affiliation, outcast, Reason} ->
 					 catch send_kickban_presence(
 						 JID, Reason, "301", SD),
-					 set_affiliation(
-					   JID, outcast,
+					 set_affiliation_and_reason(
+					   JID, outcast, Reason,
 					   set_role(JID, none, SD));
 				     {JID, affiliation, A, Reason} when
-					   (A == admin) or (A == owner)->
+					   (A == admin) or (A == owner) ->
 					 SD1 = set_affiliation(JID, A, SD),
 					 SD2 = set_role(JID, moderator, SD1),
 					 send_update_presence(JID, SD2),
@@ -1508,17 +1568,17 @@ process_admin_items_set(UJID, Items, Lang, StateData) ->
 				       end
 				) of
 			      {'EXIT', ErrReason} ->
-				  io:format("MUC ITEMS SET ERR: ~p~n",
-					    [ErrReason]),
+				  ?ERROR_MSG("MUC ITEMS SET ERR: ~p~n",
+					     [ErrReason]),
 				  SD;
 			      NSD ->
 				  NSD
 			  end
 		  end, StateData, Res),
-	    io:format("MUC SET: ~p~n", [Res]),
 	    case (NSD#state.config)#config.persistent of
 		true ->
-		    mod_muc:store_room(NSD#state.room, make_opts(NSD));
+		    mod_muc:store_room(NSD#state.host, NSD#state.room,
+				       make_opts(NSD));
 		_ ->
 		    ok
 	    end,
@@ -1671,6 +1731,19 @@ can_change_ra(FAffiliation, FRole,
 	      affiliation, none)
   when (FAffiliation == owner) or (FAffiliation == admin) ->
     true;
+can_change_ra(FAffiliation, FRole,
+	      outcast, TRole,
+	      affiliation, member)
+  when (FAffiliation == owner) or (FAffiliation == admin) ->
+    true;
+can_change_ra(owner, FRole,
+	      outcast, TRole,
+	      affiliation, admin) ->
+    true;
+can_change_ra(owner, FRole,
+	      outcast, TRole,
+	      affiliation, owner) ->
+    true;
 can_change_ra(FAffiliation, FRole,
 	      none, TRole,
 	      affiliation, outcast)
@@ -1709,15 +1782,11 @@ can_change_ra(owner, FRole,
     true;
 can_change_ra(owner, FRole,
 	      admin, TRole,
-	      affiliation, member) ->
-    true;
-can_change_ra(owner, FRole,
-	      admin, TRole,
-	      affiliation, owner) ->
+	      affiliation, _Affiliation) ->
     true;
 can_change_ra(owner, FRole,
 	      owner, TRole,
-	      affiliation, admin) ->
+	      affiliation, _Affiliation) ->
     true;
 can_change_ra(FAffiliation, FRole,
 	      TAffiliation, TRole,
@@ -1749,6 +1818,22 @@ can_change_ra(FAffiliation, FRole,
 	      role, moderator)
   when (FAffiliation == owner) or (FAffiliation == admin) ->
     true;
+can_change_ra(FAffiliation, FRole,
+	      owner, moderator,
+	      role, visitor) ->
+    false;
+can_change_ra(owner, FRole,
+	      TAffiliation, moderator,
+	      role, visitor) ->
+    true;
+can_change_ra(FAffiliation, FRole,
+	      admin, moderator,
+	      role, visitor) ->
+    false;
+can_change_ra(admin, FRole,
+	      TAffiliation, moderator,
+	      role, visitor) ->
+    true;
 can_change_ra(FAffiliation, FRole,
 	      owner, moderator,
 	      role, participant) ->
@@ -1971,7 +2056,10 @@ get_config(Lang, StateData) ->
 	       [{xmlcdata,
 		 translate:translate(
 		   Lang, "You need an x:data capable client to configure room")}]}, 
-	      {xmlelement, "x", [{"xmlns", ?NS_XDATA}], Res}], StateData}.
+	      {xmlelement, "x", [{"xmlns", ?NS_XDATA},
+				 {"type", "form"}],
+	       Res}],
+     StateData}.
 
 
 
@@ -2041,9 +2129,9 @@ change_config(Config, StateData) ->
     case {(StateData#state.config)#config.persistent,
 	  Config#config.persistent} of
 	{_, true} ->
-	    mod_muc:store_room(NSD#state.room, make_opts(NSD));
+	    mod_muc:store_room(NSD#state.host, NSD#state.room, make_opts(NSD));
 	{true, false} ->
-	    mod_muc:forget_room(NSD#state.room);
+	    mod_muc:forget_room(NSD#state.host, NSD#state.room);
 	{false, false} ->
 	    ok
 	end,
@@ -2128,7 +2216,7 @@ destroy_room(DEls, StateData) ->
       end, ?DICT:to_list(StateData#state.users)),
     case (StateData#state.config)#config.persistent of
 	true ->
-	    mod_muc:forget_room(StateData#state.room);
+	    mod_muc:forget_room(StateData#state.host, StateData#state.room);
 	false ->
 	    ok
 	end,
