@@ -3,12 +3,12 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : 
 %%% Created :  2 Jan 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: mod_vcard_ldap.erl,v 1.1 2004/07/30 21:09:55 aleksey Exp $
+%%% Id      : $Id: mod_vcard_ldap.erl 329 2005-04-26 11:22:04Z alexey $
 %%%----------------------------------------------------------------------
 
 -module(mod_vcard_ldap).
 -author('alexey@sevcom.net').
--vsn('$Revision: 1.1 $ ').
+-vsn('$Revision: 329 $ ').
 
 -behaviour(gen_mod).
 
@@ -29,7 +29,9 @@ start(Opts) ->
     gen_iq_handler:add_iq_handler(ejabberd_sm, ?NS_VCARD,
 				  ?MODULE, process_sm_iq, IQDisc),
     LDAPServers = ejabberd_config:get_local_option(ldap_servers),
-    eldap:start_link("mod_vcard_ldap", LDAPServers, 389, "", ""),
+    RootDN = ejabberd_config:get_local_option(ldap_rootdn),
+    Password = ejabberd_config:get_local_option(ldap_password),			
+    eldap:start_link("mod_vcard_ldap", LDAPServers, 389, RootDN, Password),
     Host = gen_mod:get_opt(host, Opts, "vjud." ++ ?MYNAME),
     Search = gen_mod:get_opt(search, Opts, true),
     register(ejabberd_mod_vcard_ldap, spawn(?MODULE, init, [Host, Search])).
@@ -84,7 +86,7 @@ process_local_iq(_From, _To, #iq{type = Type, lang = Lang, sub_el = SubEl} = IQ)
 				 translate:translate(
 				   Lang,
 				   "Erlang Jabber Server\n"
-				   "Copyright (c) 2002-2004 Alexey Shchepin")}]},
+				   "Copyright (c) 2002-2005 Alexey Shchepin")}]},
 			      {xmlelement, "BDAY", [],
 			       [{xmlcdata, "2002-11-16"}]}
 			     ]}]}
@@ -359,7 +361,7 @@ iq_get_vcard(Lang) ->
       [{xmlcdata, translate:translate(
 		    Lang,
 		    "ejabberd vCard module\n"
-		    "Copyright (c) 2003-2004 Alexey Shchepin")}]}].
+		    "Copyright (c) 2003-2005 Alexey Shchepin")}]}].
 
 find_xdata_el({xmlelement, _Name, _Attrs, SubEls}) ->
     find_xdata_el1(SubEls).
@@ -399,7 +401,7 @@ search_result(Lang, JID, Data) ->
        ?LFIELD("Organization Unit", "orgunit")
       ]}] ++ lists:map(fun(E) -> 
 			       record_to_item(E#eldap_entry.attributes)
-		       end, search(Data)).
+		       end, search(JID#jid.lserver, Data)).
 
 -define(FIELD(Var, Val),
 	{xmlelement, "field", [{"var", Var}],
@@ -487,15 +489,17 @@ record_to_item(Attributes) ->
     FList = [X || X <- List, X /= none],
     {xmlelement, "item", [],FList}.
 
-search(Data) ->
+search(LServer, Data) ->
     Filter = make_filter(Data),
     Base = ejabberd_config:get_local_option(ldap_base),
     UIDAttr = ejabberd_config:get_local_option(ldap_uidattr),
-    case eldap:search("mod_vcard_ldap",[{base,Base},
+    case eldap:search("mod_vcard_ldap",[{base, Base},
 					{filter, Filter},
 					{attributes, []}]) of
 	#eldap_search_result{entries = E} ->
-	    [X || X <- E, ejabberd_auth:is_user_exists(ldap_get_value(X,UIDAttr)) ];
+	    [X || X <- E,
+		  ejabberd_auth:is_user_exists(
+		    ldap_get_value(X, UIDAttr), LServer)];
 	_ ->
 	    ?ERROR_MSG("~p", ["Bad search"])
     end.
