@@ -3,12 +3,12 @@
 %%% Author  : Alexey Shchepin <alexey@sevcom.net>
 %%% Purpose : 
 %%% Created : 28 Feb 2004 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: ejabberd_web.erl,v 1.12 2004/05/22 19:48:34 aleksey Exp $
+%%% Id      : $Id: ejabberd_web.erl 307 2005-04-17 18:08:34Z tmallard $
 %%%----------------------------------------------------------------------
 
 -module(ejabberd_web).
 -author('alexey@sevcom.net').
--vsn('$Revision: 1.12 $ ').
+-vsn('$Revision$ ').
 
 %% External exports
 -export([make_xhtml/1,
@@ -50,20 +50,20 @@ make_xhtml(Els) ->
 
 
 process_get({_, true},
-	    #request{user = User,
+	    #request{us = US,
 		     path = ["admin" | RPath],
 		     q = Query,
 		     lang = Lang} = Request) ->
-    if
-	User /= undefined ->
-	    case acl:match_rule(configure, jlib:make_jid(User, ?MYNAME, "")) of
+    case US of
+	{User, Server} ->
+	    case acl:match_rule(configure, jlib:make_jid(User, Server, "")) of
 		deny ->
 		    {401, [], make_xhtml([?XC("h1", "Not Allowed")])};
 		allow ->
 		    ejabberd_web_admin:process_admin(
 		      Request#request{path = RPath})
 	    end;
-	true ->
+	undefined ->
 	    {401,
 	     [{"WWW-Authenticate", "basic realm=\"ejabberd\""}],
 	     ejabberd_web:make_xhtml([{xmlelement, "h1", [],
@@ -71,10 +71,10 @@ process_get({_, true},
     end;
 
 process_get({true, _},
-	    #request{user = User,
+	    #request{us = _US,
 		     path = ["http-poll" | RPath],
-		     q = Query,
-		     lang = Lang} = Request) ->
+		     q = _Query,
+		     lang = _Lang} = Request) ->
     ejabberd_http_poll:process_request(Request#request{path = RPath});
 
 process_get(_, _Request) ->
